library IEEE;
use IEEE.STD_LOGIC_1164.all;

package EdgeDet is

	subtype tBuffer is std_logic_vector(1 downto 0);
	constant tBuffer_InitLow : tBuffer := "00";
	constant tBuffer_InitHigh : tBuffer := "11";
	
	procedure setNext ( signal b : inout tBuffer; signal n : in std_logic );
	
	function risingEdge ( b : in tBuffer ) return boolean;
	function fallingEdge ( b : in tBuffer ) return boolean;
	function anyEdge ( b : in tBuffer ) return boolean;
	function stableHigh ( b : in tBuffer ) return boolean;
	function stableLow ( b : in tBuffer ) return boolean;

end EdgeDet;

package body EdgeDet is

	procedure setNext ( signal b : inout tBuffer; signal n : in std_logic ) is
	begin
		b <= b(0) & n;
	end setNext;
	
	function risingEdge ( b : in tBuffer ) return boolean is
	begin
		return (b = "01");
	end risingEdge;
	
	function fallingEdge ( b : in tBuffer ) return boolean is
	begin
		return (b = "10");
	end fallingEdge;

	function anyEdge ( b : in tBuffer ) return boolean is
	begin
		return risingEdge(b) or fallingEdge(b);
	end anyEdge;

	function stableHigh ( b : in tBuffer ) return boolean is
	begin
		return (b = "11");
	end stableHigh;
	
	function stableLow ( b : in tBuffer ) return boolean is
	begin
		return (b = "00");
	end stableLow;

end EdgeDet;
